const Lang = imports.lang;
const Clutter = imports.gi.Clutter;
const GLib = imports.gi.GLib;
const Meta = imports.gi.Meta;
const Atspi = imports.gi.Atspi;
const EdgeDragAction = imports.ui.edgeDragAction;

let rightClickAction;

const RightClickAction = new Lang.Class({
	Name: 'RightClickAction',
	Extends: Clutter.GestureAction,

	_init: function() {
		this.parent();

		this._threshold = Clutter.Settings.get_default().dndDragThreshold;
		this._duration = Clutter.Settings.get_default().longPressDuration;
		this._edgeThreshold = EdgeDragAction.EDGE_THRESHOLD;

		Atspi.init();
		
		global.display.connect('grab-op-begin', Lang.bind(this, function() {
			this.cancel();
		}));
		
		global.display.connect('grab-op-end', Lang.bind(this, function() {
			this._isWindowPicked = false;
			
			this.cancel();
		}));
	},
	
	vfunc_gesture_prepare: function(actor) {
		if (this.get_n_current_points() == 1) {
			[x, y] = this.get_press_coords(0);
			
			let pickedActor = global.stage.get_actor_at_pos(Clutter.PickMode.ALL, x, y);
			this._isWindowPicked = pickedActor.get_parent() instanceof Meta.WindowActor; 
			
			if (!this._isWindowPicked) {
				this.cancel();
				
				return false;
			}
			
			let monitorIndex = global.screen.get_current_monitor();
			let monitorRect = global.screen.get_monitor_geometry(monitorIndex);
			
			this._edgeStart = false;
			
			if ((x < monitorRect.x + this._edgeThreshold) ||
				(x > monitorRect.x + monitorRect.width - this._edgeThreshold) ||
				(y < monitorRect.y + this._edgeThreshold) ||
				(y > monitorRect.y + monitorRect.height - this._edgeThreshold))
				this._edgeStart = true;
			
			this._pressX = Math.floor(x);
			this._pressY = Math.floor(y);
			this._dragStarted = false;
			this._activated = false;
			this._isActive = true;
		}
		else {
			this._isActive = false;
			
			this._stop();
		}

		return true;
	},

	vfunc_gesture_begin: function(actor) {
		if (this._isActive) {
			this._timeoutId = GLib.timeout_add(GLib.PRIORITY_DEFAULT,
											   this._duration,
											   Lang.bind(this, function() {
				Atspi.generate_mouse_event(this._pressX, this._pressY, 'b3c');
	
				this._timeoutId = null;
				this._activated = true;
			}));
		}

		return true;
	},

	vfunc_gesture_progress: function(actor) {
		if (this._isActive) {
			[x, y] = this.get_motion_coords(0);
			this._motionX = Math.floor(x);
			this._motionY = Math.floor(y);
			
			if (this._dragStarted)
				Atspi.generate_mouse_event(this._motionX, this._motionY, 'abs');
			else if (Math.abs(this._motionX - this._pressX) > this._threshold ||
					 Math.abs(this._motionY - this._pressY) > this._threshold) {
				if (this._edgeStart)
					return false;
				
				this._remove_timeout();
				
				if (!this._activated)
					Atspi.generate_mouse_event(this._pressX, this._pressY, 'b1p');
	
				this._dragStarted = true;
			}
		}

		return true;
	},

	vfunc_gesture_cancel: function(actor) {
		this._stop();
	},
	
	_stop: function() {
		this._remove_timeout();
		
		if (!this._isWindowPicked)
			return;

		if (this._activated && this._dragStarted || 
			!this._activated && !this._dragStarted &&
			this._isActive && !this._edgeStart)
			Atspi.generate_mouse_event(this._motionX, this._motionY, 'b1c');
		else if (!this._activated && this._dragStarted)
			Atspi.generate_mouse_event(this._motionX, this._motionY, 'b1r');
	},

	_remove_timeout: function() {
		if (this._timeoutId) {
			GLib.source_remove(this._timeoutId);
			
			this._timeoutId = null;
		}
	}
});

function enable() {
	rightClickAction = new RightClickAction();
	global.stage.add_action(rightClickAction);
}

function disable() {
	global.stage.remove_action(rightClickAction);
}
